MODULE wavesolve
  USE atomdata
  USE globalmath

  IMPLICIT NONE

CONTAINS

  !**********************************************************************
  !      subroutine unboundsch(Grid,Pot,l,energy,wfn,nodes)
  !  pgm to solve radial schroedinger equation for unbound states
  !    at energy 'energy' and at angular momentum l
  !
  !    with potential rv/r, given in uniform mesh of n points
  !   r=i*h, i=1,...n-1 ;assuming p(r)=C*r**(l+1)*polynomial(r) for r==0;
  !                               p((n+1)*h)=0
  !  nz=nuclear charge
  !
  !  uses Noumerov algorithm
  !
  !  For l=0,1 corrections are needed to approximate wfn(r=0)
  !     These depend upon:
  !         e0 (current guess of energy eigenvalue)
  !         l,nz
  !         v(0) == v0 electronic potential at r=0
  !         v'(0) == v0p derivative of electronic potential at r=0
  !
  ! also returns node == number of nodes for calculated state
  !************************************************************************
  SUBROUTINE unboundsch(Grid,Pot,nr,l,energy,wfn,nodes)
    TYPE(GridInfo), INTENT(IN)  :: Grid
    TYPE(PotentialInfo), INTENT(IN)  :: Pot
    INTEGER, INTENT(IN) :: nr,l
    REAL(8), INTENT(IN) :: energy
    REAL(8), INTENT(INOUT) :: wfn(:)
    INTEGER, INTENT(INOUT) :: nodes

    REAL(8), POINTER :: rv(:)
    REAL(8), ALLOCATABLE ::  a(:), b(:)
    REAL(8) :: v0, v0p,h,angm,eh2,ssl,ssd
    REAL(8) :: b0p0,c1,c2,c3,fact,scale
    INTEGER :: n,nz,i,j,k,ierr

    n=Grid%n
    IF (nr > n) THEN
       WRITE(6,*) 'Error in unboundsch -- nr > n', nr,n
       STOP
    ENDIF
    h=Grid%h
    nz=Pot%nz
    v0=Pot%v0
    v0p=Pot%v0p
    rv=>Pot%rv
    ALLOCATE(a(n),b(n),stat=ierr)
    IF (ierr/=0) THEN
       WRITE(6,*) 'Allocation error in unboundsch',ierr,n
       STOP
    ENDIF
    angm=l*(l+1)
    eh2=energy*h*h
    DO i=1,nr
       a(i)=angm/(i*i)+h*rv(i)/i
    ENDDO
    DO  i=1,nr
       b(i)=-1.2d0+0.1d0*a(i)
    ENDDO
    ssl=0.1d0
    ssd=1.d0
    DO  i=1,nr
       a(i)=2.4d0+a(i)
    ENDDO
    !
    !  start outward integration
    !    correct behavior near r=0
    b0p0=0
    IF (l<=1) THEN
       c1=-nz/(l+1.d0)
       c2=((v0-energy)-2*nz*c1)/(4*l+6.d0)
       c3=(v0p+(v0-energy)*c1-2*nz*c2)/(6*l+12.d0)
       fact=1.d0+h*(c1+h*(c2+h*c3))
       IF (l.EQ.0) THEN
          b0p0=-0.2d0*h*nz/fact
       ENDIF
       IF (l.EQ.1) THEN
          b0p0=0.2d0/fact
       ENDIF
    ENDIF
    nodes=0
    wfn(1)=h**(l+1)
    wfn(2)=(a(1)+b0p0-eh2*ssd)*wfn(1)/(eh2*ssl-b(2))
    DO j=3,nr
       wfn(j)=((a(j-1)-eh2*ssd)*wfn(j-1)+                               &
            (b(j-2)-eh2*ssl)*wfn(j-2))/(eh2*ssl-b(j))
       IF (wfn(j)*wfn(j-1).LE.0.d0) nodes=nodes+1
    ENDDO
    !
    ! normalize to unity within integration range
    !
    scale=1.d0/overlap(nr,h,wfn,wfn)
    scale=DSIGN(SQRT(scale),wfn(nr-2))
    wfn(1:nr)=wfn(1:nr)*scale  

    DEALLOCATE(a,b)
  END SUBROUTINE unboundsch


END MODULE wavesolve

